@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,18 +50,20 @@
 package com.openexchange.passwordchange.database.impl;
 
 import static com.openexchange.passwordchange.database.services.DPWServiceRegistry.getServiceRegistry;
+import static com.openexchange.tools.sql.DBUtils.autocommit;
+import static com.openexchange.tools.sql.DBUtils.closeSQLStuff;
+import static com.openexchange.tools.sql.DBUtils.rollback;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
-import java.sql.ResultSet;
 import java.sql.SQLException;
 import com.openexchange.database.DBPoolingException;
 import com.openexchange.databaseold.Database;
+import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.ldap.User;
 import com.openexchange.groupware.ldap.UserException;
 import com.openexchange.passwordchange.PasswordChangeEvent;
 import com.openexchange.passwordchange.PasswordChangeService;
 import com.openexchange.server.ServiceException;
-import com.openexchange.tools.sql.DBUtils;
 import com.openexchange.user.UserService;
 
 /**
@@ -71,11 +73,6 @@ import com.openexchange.user.UserService;
  */
 public final class DatabasePasswordChange extends PasswordChangeService {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(DatabasePasswordChange.class);
-
-    /**
-     * Initializes a new {@link DatabasePasswordChange}
-     */
     public DatabasePasswordChange() {
         super();
     }
@@ -83,57 +80,72 @@ public final class DatabasePasswordChange extends PasswordChangeService {
     @Override
     protected void update(final PasswordChangeEvent event) throws UserException {
         final String encodedPassword;
+        final Context ctx = event.getContext();
         {
             final UserService userService = getServiceRegistry().getService(UserService.class);
             if (userService == null) {
                 throw new UserException(new ServiceException(ServiceException.Code.SERVICE_UNAVAILABLE, UserService.class.getName()));
             }
-            final User user = userService.getUser(event.getSession().getUserId(), event.getContext());
-            /*
-             * Get encoded version of new password
-             */
+            final User user = userService.getUser(event.getSession().getUserId(), ctx);
+            // Get encoded version of new password
             encodedPassword = getEncodedPassword(user.getPasswordMech(), event.getNewPassword());
         }
-        /*
-         * Update database
-         */
+        // Update database
         final Connection writeCon;
         try {
-            writeCon = Database.get(event.getContext(), true);
+            writeCon = Database.get(ctx, true);
+            writeCon.setAutoCommit(false);
         } catch (final DBPoolingException e) {
             throw new UserException(e);
+        } catch (final SQLException e) {
+            throw new UserException(UserException.Code.SQL_ERROR, e, e.getMessage());
         }
         try {
-            writeCon.setAutoCommit(false);
-            update(writeCon, encodedPassword, event.getSession().getUserId(), event.getContext().getContextId());
+            update(writeCon, encodedPassword, event.getSession().getUserId(), ctx.getContextId());
+            deleteAttr(writeCon, event.getSession().getUserId(), ctx.getContextId());
             writeCon.commit();
         } catch (final SQLException e) {
-            DBUtils.rollback(writeCon);
-            throw new UserException(UserException.Code.SQL_ERROR, e);
+            rollback(writeCon);
+            throw new UserException(UserException.Code.SQL_ERROR, e, e.getMessage());
+        } catch (final Exception e) {
+            rollback(writeCon);
+            throw new UserException(UserException.Code.SQL_ERROR, e, e.getMessage());
         } finally {
-            try {
-                writeCon.setAutoCommit(true);
-            } catch (final SQLException e) {
-                LOG.error("Problem setting autocommit to true.", e);
-            }
-            Database.back(event.getContext(), true, writeCon);
+            autocommit(writeCon);
+            Database.back(ctx, true, writeCon);
         }
     }
 
-    private static final String SQL_UPDATE = "UPDATE user SET userPassword = ? WHERE cid = ? AND id = ?";
+    private static final String SQL_UPDATE = "UPDATE user SET userPassword = ?, shadowLastChange = ? WHERE cid = ? AND id = ?";
 
     private void update(final Connection writeCon, final String encodedPassword, final int userId, final int cid) throws SQLException {
         PreparedStatement stmt = null;
-        final ResultSet result = null;
         try {
             stmt = writeCon.prepareStatement(SQL_UPDATE);
             int pos = 1;
             stmt.setString(pos++, encodedPassword);
+            stmt.setInt(pos++,(int)(System.currentTimeMillis()/1000));
             stmt.setInt(pos++, cid);
             stmt.setInt(pos++, userId);
             stmt.executeUpdate();
         } finally {
-            DBUtils.closeSQLStuff(result, stmt);
+            closeSQLStuff(stmt);
+        }
+    }
+
+    private static final String SQL_DELETE = "DELETE FROM user_attribute WHERE cid = ? AND id = ? AND name = ?";
+
+    private void deleteAttr(final Connection writeCon, final int userId, final int cid) throws SQLException {
+        PreparedStatement stmt = null;
+        try {
+            stmt = writeCon.prepareStatement(SQL_DELETE);
+            int pos = 1;
+            stmt.setInt(pos++, cid);
+            stmt.setInt(pos++, userId);
+            stmt.setString(pos++, "passcrypt");
+            stmt.executeUpdate();
+        } finally {
+            closeSQLStuff(stmt);
         }
     }
 }
