@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -99,10 +99,10 @@ public final class DatabasePasswordChangeActivator extends DeferredActivator {
             final ServiceRegistry registry = getServiceRegistry();
             registry.clearRegistry();
             final Class<?>[] classes = getNeededServices();
-            for (int i = 0; i < classes.length; i++) {
-                final Object service = getService(classes[i]);
+            for (final Class<?> classe : classes) {
+                final Object service = getService(classe);
                 if (null != service) {
-                    registry.addService(classes[i], service);
+                    registry.addService(classe, service);
                 }
             }
         }
